/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.kde.baloo.main.xml -a mainadaptor
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mainadaptor.h"

/*
 * Implementation of adaptor class MainAdaptor
 */

MainAdaptor::MainAdaptor(Baloo::MainHub *parent)
    : QDBusAbstractAdaptor(parent)
    , mainHub(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MainAdaptor::~MainAdaptor()
{
    // destructor
}

void MainAdaptor::quit()
{
    mainHub->quit();
}

void MainAdaptor::updateConfig()
{
    mainHub->updateConfig();
}

